#!/bin/sh
# Automate the build process of FreeNAS
#
# This will do a complete and full update of ALL files
# and build the required files.

alias cls='echo -e "\033[2J"'

# Colors
# To make these work you must include echo -e switch
NORMAL='\e[0m'
RED='\e[1;31m'
GREEN='\e[1;32m'
YELLOW='\e[1;33m'
BLUE='\e[1;34m'
MAGENTA='\e[1;35m'
CYAN='\e[1;36m'
GREY='\e[1;37m'
BOLD='\e[1m'

# The FREEBSD_CVSUP_HOST must point to a repository. Google "cvsup freebsd servers" for a listing
# of available servers.
export FREEBSD_CVSUP_HOST=cvsup8.freebsd.org
# LOCALCVSUP is the local CVSUP Mirror or can be an alternate mirror site
export LOCALCVSUP=192.168.1.56
export timestopgetsvn=0
export timestopfirstfile=0
export timestopsecondfile=0

# enable automatic iso creation
export createiso=1

# Command line switches are listed below
switch=$1
switch2=$2
switch3=$3
switch4=$4
switch5=$5



# MAIN FUNCTION ROUTINES #

getsvn ()
{
	echo ""
	echo -e "${CYAN}Updating SVN File to Current Revision"
	cd /usr/local/freenas/
	svn co https://freenas.svn.sourceforge.net/svnroot/freenas/trunk/
	echo "SVN Files Updated"
	timestopgetsvn=$(date +"%s")	
}

createsha ()
{
	echo -e "${YELLOW}Create SHA256 for images"
	echo ""
	let test="0" > null
	if [ -d /usr/local/freenas/trunk/obj.amd64/ ]; then
		xzpath="/usr/local/freenas/trunk/obj.amd64/"
		cd "$xzpath"
		let test="1" > null
		echo ""
		echo "AMD64 Files"
		ls *.xz > shatemp.txt
		rm SHA256_Hash_Values.txt 2> NUL
		cat shatemp.txt | 
		(
		while read line
		do
		#echo $line
		sha256 $line >> SHA256_Hash_Values.txt
		done
		)
		cat SHA256_Hash_Values.txt
		echo ""
		echo "The SHA256 is recorded in SHA265_Hash_Values.txt at ${xzpath}"
		echo ""
		rm shatemp.txt >> NUL

	fi
	if [ -d /usr/local/freenas/trunk/obj.i386/ ]; then
		xzpath="/usr/local/freenas/trunk/obj.i386/"
		cd "$xzpath"
		let test="1" > null
		echo ""
		echo "i386 Files"
		ls *.xz > shatemp.txt
		rm SHA256_Hash_Values.txt 2> NUL
		cat shatemp.txt | 
		(
		while read line
		do
		#echo $line
		sha256 $line >> SHA256_Hash_Values.txt
		done
		)
		cat SHA256_Hash_Values.txt
		echo ""
		echo "The SHA256 is recorded in SHA265_Hash_Values.txt at ${xzpath}"
		echo ""
		rm shatemp.txt >> NUL
	fi
	if [ $test = "0" ]; then
		echo -e "${RED}No files are available in '/usr/local/freenas/trunk/obj.amd64/' or in"
		echo -e "'usr/local/freenas/trunk/obj.i386/'${NORMAL}"
	fi
	let test="0" > null

}

buildfiles ()
{
	cd /usr/local/freenas/trunk/
	if [ "$switch2" = "-386" ]
		then
			echo -e "${MAGENTA}Building First File (32-bit)"
			env FREENAS_ARCH=i386 sh build/do_build.sh
			timestopfirstfile=$(date +"%s")
				if [ createiso != 0 ]
				then
				sh build/create_iso.sh
				timestopsecondfile=$(date +"%s")
				fi
			echo "32-bit File Build Complete"
		else
			echo -e "${MAGENTA}Building First File (64-bit)"
			sh build/do_build.sh
			timestopfirstfile=$(date +"%s")
				if [ createiso != 0 ]
				then
				sh build/create_iso.sh
				timestopsecondfile=$(date +"%s")
				fi
			echo "64-bit File Build Complete"		
	fi
}

updatebuildfiles ()
{
	cd /usr/local/freenas/trunk/
	if [ "$switch2" = "-386" ]
		then
			echo -e "${MAGENTA}Building First File (32-bit)"
			env FREENAS_ARCH=i386 sh build/do_build.sh -fu
			timestopfirstfile=$(date +"%s")
				if [ createiso != 0 ]
				then
				sh build/create_iso.sh
				timestopsecondfile=$(date +"%s")
				fi
			echo "32-bit File Build Complete"
		else
			echo -e "${MAGENTA}Building First File (64-bit)"
			sh build/do_build.sh -fu
			timestopfirstfile=$(date +"%s")
				if [ createiso != 0 ]
				then
				sh build/create_iso.sh
				timestopsecondfile=$(date +"%s")
				fi
			echo "64-bit File Build Complete"		
	fi
}

updatebuildfilesbranch ()
{
	cd /usr/local/freenas/trunk/
	if [ "$switch2" = "-386" ]
		then
			echo -e "${MAGENTA}Building First File (32-bit)"
			env FREENAS_ARCH=i386 sh build/do_build.sh -b
			timestopfirstfile=$(date +"%s")
				if [ createiso != 0 ]
				then
				sh build/create_iso.sh
				timestopsecondfile=$(date +"%s")
				fi
			echo "32-bit File Build Complete"
		else
			echo -e "${MAGENTA}Building First File (64-bit)"
			sh build/do_build.sh -b
			timestopfirstfile=$(date +"%s")
				if [ createiso != 0 ]
				then
				sh build/create_iso.sh
				timestopsecondfile=$(date +"%s")
				fi
			echo "64-bit File Build Complete"		
	fi
}


deletesvn ()
{
	echo "Deleting SVN files..."
	cd /usr/local/freenas/trunk
	if [ -d /usr/local/freenas/trunk/.svn/ ]; then
		chflags -R 0 .svn/*
		rm -r .svn
			if [ $? != 0 ]; then
			echo "'.svn' files not deleted"
			else
			echo "'.svn' files deleted"
			fi
	else
	echo "'.svn' files not present"
	fi
	if [ -d /usr/local/freenas/trunk/build/ ]; then
		chflags -R 0 build/*
		rm -r build
			if [ $? != 0 ]; then
			echo ""
			echo "'build' files not deleted"
			else
			echo "'build' files deleted"
			fi
	else
	echo "'build' files not present"
	fi
	if [ -d /usr/local/freenas/trunk/gui/ ]; then
		chflags -R 0 gui/*
		rm -r gui
			if [ $? != 0 ]; then
			echo ""
			echo "'gui' files not deleted"
			else
			echo "'gui' files deleted"
			fi
	else
	echo "'gui' files not present"
	fi
	if [ -d /usr/local/freenas/trunk/nanobsd/ ]; then
		chflags -R 0 nanobsd/*
		rm -r nanobsd
			if [ $? != 0 ]; then
			echo ""
			echo "'nanobsd' files not deleted"
			else
			echo "'nanobsd' files deleted"
			fi
	else
	echo "'nanobsd' files not present"
	fi
	if [ -d /usr/local/freenas/trunk/patches/ ]; then
		chflags -R 0 patches/*
		rm -r patches
			if [ $? != 0 ]; then
			echo ""
			echo "'patches' files not deleted"
			else
			echo "'patches' files deleted"
			fi
	else
	echo "'patches' files not present"
	fi
	if [ -d /usr/local/freenas/trunk/tools/ ]; then
		chflags -R 0 tools/*
		rm -r tools
			if [ $? != 0 ]; then
			echo ""
			echo "'tools' not deleted"
			else
			echo "'tools' files deleted"
			fi
	else
	echo "'tools' files not present"
	fi
	if [ -f /usr/local/freenas/trunk/README ]; then
		rm README
			if [ $? != 0 ]; then
			echo ""
			echo "'README' file not deleted"
			else
			echo "'README' file deleted"
			fi
	else
	echo "'README' file not present"
	fi
cd /usr/local/freenas
}

quit ()
{
	echo ""
	timestop=$(date +"%s")
	timetotal=$(($timestop - $timestart))
	let totalhours=18000+timetotal >> NUL

	if [ "$timestopgetsvn" != "0" ]
	then
	timetotalgetsvn=$(($timestopgetsvn - $timestart))
	let totalhoursgetsvn=18000+timetotalgetsvn >> NUL
	echo -e "${CYAN}Download SVN Files Time: " $(date -r $totalhoursgetsvn "+%H:%M:%S")
	fi

	if [ "$timestopfirstfile" != "0" ]
	then
	timetotalfirstfile=$(($timestopfirstfile - $timestart))
	let totalhoursfirstfile=18000+timetotalfirstfile >> NUL
	echo -e "${CYAN}NanoBSD Build Time: " $(date -r $totalhoursfirstfile "+%H:%M:%S")
	fi

	if [ "$timestopsecondfile" != "0" ]
	then
	timetotalsecondfile=$(($timestopsecondfile - $timestopfirstfile))
	let totalhourssecondfile=18000+timetotalsecondfile >> NUL
	echo -e "${CYAN}iso Build Time: " $(date -r $totalhourssecondfile "+%H:%M:%S")
	fi

	echo -e "${CYAN}Elapsed Time: " $(date -r $totalhours "+%H:%M:%S") ${NORMAL}
	exit 0
}

# Start of the program
cls
timestart=$(date +"%s")
	echo ""
#	echo "12345678901234567890123456789012345678901234567890123456789012345678901234567890"
	echo "=============================================================================="
	echo "FreeNAS Building Script - by Joe Schmuckatelli              (30 December 2011)"
	echo ""

# Build current files, no updates
if [ "$switch" = "-b" ]
	then
	buildfiles
	createsha
	quit
fi

if [ "$switch" = "-bx" ]
	then
	export FREEBSD_CVSUP_HOST=$LOCALCVSUP
	buildfiles
	createsha
	quit
fi

# Build current files, update BSD source and port files.
if [ "$switch" = "-bu" ]
	then
	updatebuildfiles
	createsha
	quit
fi

if [ "$switch" = "-bbu" ]
	then
	updatebuildfilesbranch
	createsha
	quit
fi

if [ "$switch" = "-bux" ]
	then
	export FREEBSD_CVSUP_HOST=$LOCALCVSUP
	updatebuildfiles
	createsha
	quit
fi

if [ "$switch" = "-bbux" ]
	then
	export FREEBSD_CVSUP_HOST=$LOCALCVSUP
	updatebuildfilesbranch
	createsha
	quit
fi

# Build Specific Revision
# This results in deleting Trunk so you start clean
if [ "$switch" = "-buildrev" ]
	then
	if [ "$switch2" != "" ]
		then
		if [ "$switch3" = "-386" ]
			then
			if [ "$switch4" != "" ]
				then
				export FREEBSD_CVSUP_HOST=cvsup$switch4.us.freebsd.org
			fi			
			else
			if [ "$switch3" != "" ]
				then
				export FREEBSD_CVSUP_HOST=cvsup$switch3.us.freebsd.org
			fi
		fi
		echo "Deleting Trunk Files"
		cd /usr/local/freenas/
		chflags -R 0 trunk/*
		rm -r trunk
			if [ $? != 0 ]; then
			echo ""
			echo "This function requires removing any previous files."
			echo "Not all files were deleted.  This normally occurs during an aborted build or"
			echo "build failure and the files are locked.  Reboot and try again."
			quit
			else
			echo "Trunk files deleted"
			fi
		echo ""
		echo "Loading SVN Trunk Files for Revision ${switch2}"
		svn co -r $switch2 https://freenas.svn.sourceforge.net/svnroot/freenas/trunk
		echo "Trunk Reloaded with fresh files"
		cd /usr/local/freenas/trunk/
		if [ "$switch3" = "-386" ]
			then
			let switch2=$3
		fi
		buildfiles
		createsha
		quit
		else
		echo "Incorrect Format."
		quit
	fi
fi
	
#Delete all build files
if [ "$switch" = "-clean" ]
	then
	echo "Cleaning Files"
	echo ""
	if [ "$switch2" = "-386" ]
		then
		echo "Deleting 386 .iso .xz .full and .sha files ..."	
		rm /usr/local/freenas/trunk/obj.i386/*.iso
		rm /usr/local/freenas/trunk/obj.i386/*.xz
	#	rm /usr/local/freenas/trunk/obj.i386/*.full
		rm /usr/local/freenas/trunk/obj.i386/SHA256_Hash_Values.txt
		else
		echo "Deleting 64-bit .iso .xz .full and .sha files ..."
		rm /usr/local/freenas/trunk/obj.amd64/*.iso
		rm /usr/local/freenas/trunk/obj.amd64/*.xz
	#	rm /usr/local/freenas/trunk/obj.amd64/*.full
		rm /usr/local/freenas/trunk/obj.amd64/SHA256_Hash_Values.txt
	fi
	quit
fi

#Delete all files related to do_build
if [ "$switch" = "-delete" ]
	then
	if [ -d /usr/local/freenas/trunk/ ]; then
	echo "Deleting Trunk Files..."
	cd /usr/local/freenas/
	chflags -R 0 trunk/*
	rm -r trunk
# Test if the rm fails to remove all files, is there a exit code to text?
		if [ $? != 0 ]
		then
		echo ""
		echo "Not all files were deleted.  This normally occurs during an aborted build or"
		echo "build failure and the files are locked.  Reboot and try again."
		else
		echo "Trunk files deleted."
		fi
	quit
	fi
	echo "No trunk files present."
	quit
fi

#Delete object directory
if [ "$switch" = "-deleteobj" ]
	then
	if [ -d /usr/local/freenas/trunk/obj.i386/ ]; then
	echo "Deleting obj.i386 Files"
	cd /usr/local/freenas/trunk
	chflags -R 0 obj.i386/*
	rm -r obj.i386
		if [ $? != 0 ]; then
		echo ""
		echo "Not all files were deleted.  This normally occurs during an aborted build or"
		echo "build failure and the files are locked.  Reboot and try again."
		else
		echo "Obj.i386 files deleted"
		fi
	fi

	if [ -d /usr/local/freenas/trunk/obj.amd64/ ]; then
	echo "Deleting obj.amd64 Files"
	cd /usr/local/freenas/trunk
	chflags -R 0 obj.amd64/*
	rm -r obj.amd64
		if [ $? != 0 ]; then
		echo ""
		echo "Not all files were deleted.  This normally occurs during an aborted build or"
		echo "build failure and the files are locked.  Reboot and try again."
		else
		echo "Obj.amd64 files deleted"
		fi
	fi
	quit
fi

# Delete all SVN related download
if [ "$switch" = "-deletesvn" ]
	then
	deletesvn
	quit
fi

# HELP FILE
if [ "$switch" = "-h" ]
	then
#	echo "12345678901234567890123456789012345678901234567890123456789012345678901234567890"
	echo ""
	echo -e "${BOLD}                      Help File for the building script.${NORMAL}"
	echo ""
	echo ""
	echo "Description of switches (note: the switces cannot be combined)"
	echo ""
	echo " -b [-386]"
	echo "    This will only build your files, will not update any SVN files, and is"
	echo "    useful to make changes to the source code prior to compiling."
	echo ""
	echo " -bu [-386]"
	echo "    Same as -b but will update BSD source and port files."
	echo ""
	echo " -buildrev revision [-386] [US SVN Server#]"
	echo "    Specifies the revision to build.  This is handy to go back to an older"
	echo "    revision.  This will delete the Trunk directory in order to create a clean"
	echo "    build and then download and build everything."
	echo "       Example: sh fbuild.sh -buildrev 8515"
	echo ""
	echo " -clean [-386]"
	echo "    Delete all the built files ending with .iso, .xz, .sha, .full in order to"
	echo "    cleanup old files because these files do consume a lot of space."
	echo ""
	if [ "$switch2" = "-m" ]; then
	echo ""
	echo "Press 'Enter' key to continute"
	read null
	clear
	echo ""
	fi
	echo " -delete"
	echo "    Removes all files trunk files.  If this fails, reboot and try again."
	echo ""
	echo " -deleteobj"
	echo "    Removes all obj.amd64 and obj.i386 directories, great for a rebuild."
	echo ""
	echo " -deletesvn"
	echo "    Removes all FreeNAS (SVN) source code."
	echo ""
	echo " -list"
	echo "    This will list all the .xz .iso and .full files including file size."
	echo ""
	echo " -loadrev revision [US SVN Server#]"
	echo "    Same as -buildrev, only loads the SVN files, no building."
	echo ""
	echo " -s"
	echo "    This will update SVN Trunk files (the working Freenas files) and not"
	echo "    compile/build the files.  This is a way to check for new updates."
	echo ""
	echo " -sb [-386] [US SVN Server#]"
	echo "    This is the same as -s but it will compile/build the files."
	echo ""
	echo " -sbu [-386] [US SVN Server#]"
	echo "    Same as -sb but will update BSD source and port files."
	echo ""
	if [ "$switch2" = "-m" ]; then
	echo "Press 'Enter' key to continue"
	read null
	clear
	echo ""	
	fi
	echo " -sha"
	echo "    Rebuild SHA256 Hash files.  A file called SHA256_Hash_Values.txt will be"
	echo "    created for all .xz files in the /usr/local/freenas/trunk/obj.amd64 and"
	echo "    obj.i386 directories."
	echo ""
	echo " -freebsd"
	echo "    This will update FreeBSD to the most current revision and update the"
	echo "    installed ports collection.  A reboot is highly recommended."
	echo ""
	echo "*** Building Branches - This may require technical knowledge to compile."
	echo ""
	echo " -brs branch# [-u] = Download branch."
	echo "    This will delete everything and download the branch."
	echo "    -u will not delete but rather update the branch only."
	echo ""
	if [ "$switch2" = "-m" ]; then
	echo ""
	echo ""
	echo ""
	echo ""
	echo ""
	echo ""
	echo "Press 'Enter' key to continue"
	read null
	clear
	echo ""
	fi	
	echo " Notes: The default is to build a 64-bit version of this application."
	echo "    Use the -386 switch to create a 32-bit version."
	echo ""
	echo "    [US SVN Server] can be changed to any country.  Edit this file and"
	echo "    replace .us.freebsd.org thoughout the file with your preference."
	echo ""
	echo "    An Elapsed Time Meter will indicate HH:MM:SS for all operations"
	echo "    letting you know the duration of a task."
	echo ""
	echo "If a compile fails, use -delete then try again.  If it fails again then the"
	echo "source could be broken or the method to compile has changed yet again."
	echo ""
	echo ""
	exit 0
fi

#List all built files

if [ "$switch" = "-list" ]
	then
	echo "Listing Compiled Files"
	echo ""
	echo "Size    File Name"
	cd /usr/local/freenas/trunk
	files="obj.amd64/*.full obj.amd64/*.xz obj.amd64/*.iso obj.i386/*.full obj.i386/*.xz obj.i386/*.iso"
	for i in $files
	do
	if [ -f $i ]; then
	du -h $i
	fi
	done 
	quit
fi

# Loads Specific Revision
# This results in deleting Trunk so you start clean
if [ "$switch" = "-loadrev" ]
	then
	if [ "$switch2" != "" ]
		then
		if [ "$switch3" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch3.us.freebsd.org
			echo "Using cvsup${switch3}.us.freebsd.org"
		fi
		deletesvn
		echo ""
		echo "Loading SVN Trunk Files for Revision ${switch2}"
		svn co -r $switch2 https://freenas.svn.sourceforge.net/svnroot/freenas/trunk
		echo "Trunk Reloaded with fresh files"
		quit
	fi
fi

# Update SVN Trunk Files
if [ "$switch" = "-s" ]
	then
	if [ "$switch2" != "" ]
		then
		export FREEBSD_CVSUP_HOST=cvsup$switch2.us.freebsd.org
	fi
	getsvn
	quit
fi

# Update SVN Branch Files
# switch2 = branch, switch3 = update only
if [ "$switch" = "-brs" ]
	then
	if [ "$switch2" != "" ]
		then
		if [ "$switch3" != "-u" ]
		then
		echo "Deleting Any Trunk Files"
		cd /usr/local/freenas/
		if [ -d /usr/local/freenas/trunk/ ]; then
		chflags -R 0 trunk/*
		rm -r trunk
			if [ $? != 0 ]; then
			echo ""
			echo "This function requires removing any previous files to ensure a clean build."
			echo "Not all files were deleted.  This normally occurs during an aborted build or"
			echo "build failure and the files are locked.  Reboot and try again."
			quit
			else
			echo "Trunk files deleted"
			fi
		fi
		fi
	echo ""
	echo "Loading SVN Branch ${switch2} Files"
	svn co https://freenas.svn.sourceforge.net/svnroot/freenas/branches/$switch2 trunk
	echo "Trunk Reloaded with fresh branch ${switch2} files"
	quit
	else
	echo "Incorrect Format. Must specify a branch to download."
	echo "Example: sh fbuild.sh -brs 8.0.2"
	echo "  Download branch 8.0.2 using default cvsup repository."
	echo "or"
	echo "Example: sh fbuild.sh -brs 8.0.2 10"
	echo "  Download branch 8.0.2 using cvsup repository 10.us.freebsd.org."
	quit
	fi
fi

# Update SVN Trunk Files and build
if [ "$switch" = "-sb" ]
	then
	if [ "$switch2" = "-386" ]
		then
		if [ "$switch3" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch3.us.freebsd.org
		fi
		else
		if [ "$switch2" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch2.us.freebsd.org
		fi
	fi
	getsvn
	buildfiles
	createsha
	quit
fi

if [ "$switch" = "-sbx" ]
	then
	if [ "$switch2" = "-386" ]
		then
		if [ "$switch3" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch3.us.freebsd.org
		fi
		else
		if [ "$switch2" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch2.us.freebsd.org
		fi
	fi
	getsvn
	export FREEBSD_CVSUP_HOST=$LOCALCVSUP
	buildfiles
	createsha
	quit
fi

# Update SVN Trunk Files, Port and Source files, and build
if [ "$switch" = "-sbu" ]
	then
	if [ "$switch2" = "-386" ]
		then
		if [ "$switch3" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch3.us.freebsd.org
		fi
		else
		if [ "$switch2" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch2.us.freebsd.org
		fi
	fi
	getsvn
	updatebuildfiles
	createsha
	quit
fi

if [ "$switch" = "-sbux" ]
	then
	if [ "$switch2" = "-386" ]
		then
		if [ "$switch3" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch3.us.freebsd.org
		fi
		else
		if [ "$switch2" != "" ]
			then
			export FREEBSD_CVSUP_HOST=cvsup$switch2.us.freebsd.org
		fi
	fi
	getsvn
	export FREEBSD_CVSUP_HOST=$LOCALCVSUP
	updatebuildfiles
	createsha
	quit
fi

# Create SHA for build files
if [ "$switch" = "-sha" ]
	then
	createsha
	quit
fi

# Update FreeBSD
if [ "$switch" = "-freebsd" ]
	then
	echo ""
	echo "Updating FreeBSD"
	uname -a
	freebsd-update fetch install
	echo rehash
	portsnap fetch
	portsnap update
	portupgrade -a
	echo "Cleaning Up Ports Collection"
	portsclean -C
	portsclean -D
	echo "Ports are cleaned"
	echo "Update Complete!"
	echo -e "${RED}If files were updated you should reboot.${NORMAL}"
	quit
fi

# Fall Though
#	echo "12345678901234567890123456789012345678901234567890123456789012345678901234567890"
	echo -e "${RED}    No switch or invalid switch detected.${NORMAL}"
	echo "Useage: sh fbuild.sh [-switch]"
	echo " -b [-386] = Build, do not update any SVN files."	
	echo " -bu [-386] = Build, Update BSD Source & Port files."
	echo " -buildrev revision [-386] [Server#] = Builds a designated revision of FreeNAS."
	echo " -clean [-386]= Delete compiled files (.xz .iso .sha)"
	echo " -delete = Delete ALL Trunk files."
	echo " -deleteobj = Delete both obj.amd64 and obj.i386 directories if they exist."
	echo " -deletesvn = Delete all FreeNAS (SVN) source code."
	echo " -freebsd = Update FreeBSD and installed ports collection."
	echo " -h [-m] = Displays Full Help Info. -m displays one page at a time."
	echo " -list = List major build files created."
	echo " -loadrev revision [Server#] = Same as -buildrev, loads SVN files, no building."
	echo " -s [Server#] = Update SVN Trunk files only - No build."
	echo " -sb [-386] [Server#] = Update SVN Trunk files only then build."
	echo " -sbu [-386] [Server#] = Update SVN Trunk, BSD Source/Port, and build."
	echo " -sha = Create SHA256 hash for all .xz build files."
	echo ""
	echo " Building Branches (New Functions)"
	echo " -brs branch# [-u] = Download branch/update."
	echo -e " Add 'x' after any ${BOLD}build${NORMAL} option to use local server LOCALCVSUP"
	echo "  -bx or -sbx or -sbux or buildrevx."
	echo " -bbu[x] = Build Branch Update: Different build switches from trunk builds."
	echo ""
	echo " Note: Default build is 64-bit.  Switch -386 Creates 32-bit code."
	echo ""
	exit 0

# Done